/**
 * 
 */
package com.azwul.api.service.helper;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.NameValuePair;
import org.apache.commons.httpclient.UsernamePasswordCredentials;
import org.apache.commons.httpclient.auth.AuthScope;
import org.apache.commons.httpclient.cookie.CookiePolicy;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.multipart.FilePart;
import org.apache.commons.httpclient.methods.multipart.MultipartRequestEntity;
import org.apache.commons.httpclient.methods.multipart.Part;
import org.apache.commons.httpclient.methods.multipart.StringPart;

import com.azwul.api.exception.AzwulApiException;
import com.azwul.api.model.HttpConfiguration;
import com.azwul.api.model.SessionToken;
import com.azwul.api.model.result.HttpResult;

/**
 * @author Theotime Jurzak
 * 
 */
public class HttpHelper {

	protected NameValuePair[] buildParams(Map<String, String> paramsMap) {
		List<NameValuePair> paramList = new ArrayList<NameValuePair>();

		for (String paramName : paramsMap.keySet()) {
			NameValuePair param = new NameValuePair(paramName,
					paramsMap.get(paramName));
			paramList.add(param);
		}

		return paramList.toArray(new NameValuePair[0]);
	}

	public HttpResult doPost(String uri, HttpConfiguration httpConfiguration,
			SessionToken sessionToken, Map<String, String> paramsMap) {
		HttpClient client = initClient(httpConfiguration);
		PostMethod post = new PostMethod(httpConfiguration.getBaseHost() + uri);

		if (sessionToken != null) {
			post.getParams().setCookiePolicy(CookiePolicy.IGNORE_COOKIES);
			post.setRequestHeader("Cookie", sessionToken.getCookie());

			paramsMap.put("apiId", sessionToken.getApiId());
		}

		post.setRequestBody(buildParams(paramsMap));

		try {
			client.executeMethod(post);
		} catch (Exception e) {
			post.releaseConnection();
			throw new AzwulApiException(e);
		}

		HttpResult httpResult = new HttpResult();
		httpResult.setHttpMethod(post);
		httpResult.setSessionToken(sessionToken);

		return httpResult;

	}

	public HttpResult doGet(String uri, HttpConfiguration httpConfiguration,
			SessionToken sessionToken) {
		HttpClient client = initClient(httpConfiguration);

		GetMethod get = new GetMethod(httpConfiguration.getBaseHost() + uri);

		if (sessionToken != null) {
			get.getParams().setCookiePolicy(CookiePolicy.IGNORE_COOKIES);
			get.setRequestHeader("Cookie", sessionToken.getCookie());
			get.getParams().setParameter("apiId", sessionToken.getApiId());
		}

		try {
			client.executeMethod(get);
		} catch (Exception e) {
			get.releaseConnection();
			throw new AzwulApiException(e);
		}

		HttpResult httpResult = new HttpResult();
		httpResult.setHttpMethod(get);
		httpResult.setSessionToken(sessionToken);

		return httpResult;

	}

	public HttpResult doPostMultipart(String uri,
			HttpConfiguration httpConfiguration, SessionToken sessionToken,
			File file, String fileParamName, Map<String, String> paramsMap) {
		HttpClient client = initClient(httpConfiguration);

		PostMethod filePost = new PostMethod(httpConfiguration.getBaseHost()
				+ uri);
		List<Part> partList = new ArrayList<Part>();

		if (sessionToken != null) {
			filePost.getParams().setCookiePolicy(CookiePolicy.IGNORE_COOKIES);
			filePost.setRequestHeader("Cookie", sessionToken.getCookie());

			partList.add(new StringPart("apiId", sessionToken.getApiId()));
		}

		try {
			partList.add(new FilePart(fileParamName, file));
		} catch (FileNotFoundException e) {
			throw new AzwulApiException(e);
		}

		for (String paramName : paramsMap.keySet()) {
			partList.add(new StringPart(paramName, paramsMap.get(paramName)));
		}

		filePost.setRequestEntity(new MultipartRequestEntity(partList
				.toArray(new Part[0]), filePost.getParams()));

		try {
			client.executeMethod(filePost);
		} catch (Exception e) {
			filePost.releaseConnection();
			throw new AzwulApiException(e);
		}

		HttpResult httpResult = new HttpResult();
		httpResult.setHttpMethod(filePost);
		httpResult.setSessionToken(sessionToken);

		return httpResult;
	}

	protected HttpClient initClient(HttpConfiguration httpConfiguration) {
		HttpClient client = new HttpClient();

		if (httpConfiguration.getProxyConfiguration() != null) {
			client.getHostConfiguration().setProxy(
					httpConfiguration.getProxyConfiguration().getHost(),
					httpConfiguration.getProxyConfiguration().getPort());
			if (httpConfiguration.getProxyConfiguration().getUsername() != null) {
				client.getState().setProxyCredentials(
						AuthScope.ANY,
						new UsernamePasswordCredentials(httpConfiguration
								.getProxyConfiguration().getUsername(),
								httpConfiguration.getProxyConfiguration()
										.getPassword()));
			}
		}

		return client;
	}

}
