/**
 * 
 */
package com.azwul.api.service.impl;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import com.azwul.api.exception.AzwulApiException;
import com.azwul.api.model.HttpConfiguration;
import com.azwul.api.model.SessionToken;
import com.azwul.api.model.result.ClicInitResult;
import com.azwul.api.model.result.ClicLoginResult;
import com.azwul.api.model.result.SignatureResult;
import com.azwul.api.service.AzClicService;
import com.azwul.api.service.helper.HelperFactory;
import com.azwul.api.service.helper.HttpHelper;
import com.azwul.api.util.ResponseUtil;

/**
 * @author Theotime Jurzak
 * 
 */
public final class AzClicServiceImpl implements AzClicService {

	private HttpHelper httpHelper = HelperFactory.getHttpHelper();

	private HttpConfiguration httpConfiguration = new HttpConfiguration();

	public ClicInitResult doInit(String token, String secret, String email,
			String documentUrl, String callbackUrl) {
		return doInit(token, secret, email, documentUrl, null, callbackUrl,
				null);
	}

	public ClicInitResult doInit(String token, String secret, String email,
			String documentUrl, String signatureFormat, String callbackUrl,
			String responseFormat) {
		Map<String, String> params = new HashMap<String, String>();

		params.put("secret", secret);
		params.put("token", token);
		params.put("email", email);
		params.put("documentUrl", documentUrl);
		params.put("callbackUrl", callbackUrl);
		params.put("responseFormat", responseFormat);
		params.put("pdfMode", signatureFormat);

		try {
			return ResponseUtil.buildClicInitResult(httpHelper.doPost(
					"/clic/init.html", httpConfiguration, null, params));
		} catch (Exception e) {
			throw new AzwulApiException(e);
		}
	}

	public ClicLoginResult doLogin(String sessionId, File keystore,
			String password) {

		httpHelper.doGet("/clic/start.html?id=" + sessionId, httpConfiguration,
				null);

		Map<String, String> params = new HashMap<String, String>();

		params.put("clicSessionId", sessionId);
		params.put("j_password", password);

		httpHelper.doPostMultipart("/j_spring_security_check",
				httpConfiguration, null, keystore, "j_key", params);

		return ResponseUtil
				.buildClicLoginResult(httpHelper.doGet("/clic/confirm.html",
						httpConfiguration, null), null);
	}

	public SignatureResult doSign(String secret, SessionToken sessionToken) {
		Map<String, String> params = new HashMap<String, String>();
		params.put("secret", secret);

		return ResponseUtil.buildSignatureResult(httpHelper.doPost(
				"/clic/confirm.html", httpConfiguration, sessionToken, params));
	}

	/**
	 * @param httpConfiguration
	 *            the httpConfiguration to set
	 */
	public void setHttpConfiguration(HttpConfiguration httpConfiguration) {
		this.httpConfiguration = httpConfiguration;
	}

	/**
	 * @param httpHelper
	 *            the httpHelper to set
	 */
	public void setHttpHelper(HttpHelper httpHelper) {
		this.httpHelper = httpHelper;
	}

}
