/**
 * 
 */
package com.azwul.api.util;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import com.azwul.api.exception.AzwulApiException;
import com.azwul.api.model.SessionToken;
import com.azwul.api.model.result.ClicInitResult;
import com.azwul.api.model.result.ClicLoginResult;
import com.azwul.api.model.result.HttpResult;
import com.azwul.api.model.result.SignatureResult;

/**
 * @author Theotime Jurzak
 * 
 */
public final class ResponseUtil {

	public static SignatureResult buildSignatureResult(HttpResult httpResult) {
		String basicResponse = getBasicResponse(httpResult);
		SignatureResult res = new SignatureResult();

		res.setHttpResult(httpResult);
		String resSplit[] = basicResponse.split("\\s");
		if ("OK".equals(resSplit[1])) {
			res.setOk(true);
		}

		res.setSignatureId(resSplit[3]);

		return res;
	}

	public static ClicInitResult buildClicInitResult(HttpResult httpResult) {
		String basicResponse = getBasicResponse(httpResult);
		ClicInitResult res = new ClicInitResult();
		res.setHttpResult(httpResult);
		res.setBasicResponse(basicResponse);

		String resSplit[] = basicResponse.split("\\s");
		if ("OK".equals(resSplit[1])) {
			res.setOk(true);
		}

		res.setSessionId(resSplit[3]);

		return res;
	}

	public static ClicLoginResult buildClicLoginResult(HttpResult httpResult,
			SessionToken sessionToken) {
		String basicResponse = getBasicResponse(httpResult);
		ClicLoginResult res = new ClicLoginResult();
		res.setHttpResult(httpResult);
		res.setBasicResponse(basicResponse);

		String resSplit[] = basicResponse.split("\\s");
		if ("OK".equals(resSplit[1])) {
			res.setOk(true);
		}

		return res;
	}

	public static String getBasicResponse(HttpResult httpResult) {
		InputStream is = httpResult.getHttpResponse();
		BufferedReader reader = new BufferedReader(new InputStreamReader(is));
		try {
			return reader.readLine();
		} catch (IOException e) {
			throw new AzwulApiException("Error reading basic response", e);
		} finally {
			try {
				is.close();
				httpResult.releaseConnection();
			} catch (Exception ex) {
				throw new AzwulApiException("Error closing basic response", ex);
			}
		}
	}

	public static String getFullResponse(HttpResult httpResult) {
		InputStream is = httpResult.getHttpResponse();
		BufferedReader reader = new BufferedReader(new InputStreamReader(is));
		StringBuffer sb = new StringBuffer();
		try {
			String line = null;
			while ((line = reader.readLine()) != null) {
				sb.append(line + "\n");
			}
		} catch (Exception ex) {
			throw new AzwulApiException("Error reading basic full", ex);
		} finally {
			try {
				is.close();
				httpResult.releaseConnection();
			} catch (Exception ex) {
				throw new AzwulApiException("Error closing full response", ex);
			}
		}
		return sb.toString();
	}

}
